/*
 * XML Validation based on XML Schema (XSD)
 * 
 * by Alain VIZZINI 
 * ESSI - Sophia Antipolis
 * Microsoft France
 */

using System;
using System.IO;
using System.Xml;
using System.Xml.Schema;



/// <summary>
/// Summary description for Class1.
/// </summary>
class XMLValidator
{
  
  /// <summary>
  /// True if some errors occured during the validation.
  /// </summary>
  private bool errors = false;
  public bool Errors{
    get {
      return errors;
    }
  }

  /// <summary>
  /// True if some warnings occured during the validation.
  /// </summary>
  private bool warnings = false;
  public bool Warnings{
    get{
      return warnings;
    }
  }


  /// <summary>
  ///	The XML Schema (.xsd) file used to validate.
  /// </summary>
  private String xsdfile;
  public String XSDFile{
    get{
      return xsdfile;
    }
  }
  /// <summary>
  /// The XML file to validate.
  /// </summary>
  private String xmlfile;
  public String XMLFile{
    get
      {
	return xmlfile;
      }
  }
			

  /// <summary>
  /// Builds a new instance of MyXMLValidator
  /// </summary>
  /// <param name="XSDFile">XML Schema (.xsd) file to use to validate</param>
  /// <param name="XMLFile">XML file to validate</param>
  public XMLValidator (String XSDFile, String XMLFile){
    xmlfile = XMLFile;
    xsdfile = XSDFile;
  }


  /// <summary>
  /// Checks if the file is valid (concordant with the XSD file)
  /// </summary>
  /// <remarks>Sets the Warnings and Errors attributes</remarks>
  public void Validate(){
    //create a collection of XML Schema
    XmlSchemaCollection xsdColl = new XmlSchemaCollection();
    xsdColl.Add("", new XmlTextReader(XSDFile));

    XmlValidatingReader vr = new XmlValidatingReader(new XmlTextReader (XMLFile));
    vr.ValidationType = ValidationType.Schema;
    vr.Schemas.Add(xsdColl);
    vr.ValidationEventHandler += new ValidationEventHandler (XSDValidationHandler);

    while(vr.Read()) { }
  }

  /// <summary>
  /// Handler called on every ValidationEvent. Used to set Errors and Warnings flags.
  /// </summary>
  /// <param name="sender">Sender of the Event</param>
  /// <param name="args">arguments</param>
  private void XSDValidationHandler(object sender, ValidationEventArgs args)
    {
      if (args.Severity == XmlSeverityType.Warning)
	warnings = true;
      if (args.Severity == XmlSeverityType.Error)
	errors = true;
    }


  /*********************************************************************************
   * 
   * Under this point, static test purpose methods.
   * 
   *********************************************************************************/
  /// <summary>
  /// The main entry point for the application.
  /// </summary>
  public static void Main(string[] args){
    //
    // TODO: Add code to start application here
    //
    if (args.Length < 2)
      {
	Console.WriteLine ( "usage:\n"+
			    "\tAlXMLValidator schema.xsd file.xml");				
      }
    else
      {
	String XSDFile = args[0];
	String XMLFile = args[1];
	
	Console.WriteLine("Starting validation");
	
	XMLValidator xv = new XMLValidator (XSDFile, XMLFile);
	
	xv.Validate();
	
	Console.WriteLine("\nValidation finished :");
	if (!xv.errors && !xv.warnings)
	  Console.WriteLine ("\t pas d'erreur ni de warnings !");
	else
	  if (xv.errors)
	    Console.WriteLine ("\til y a eu des erreurs");
	  else
	    if (xv.warnings)
	      Console.WriteLine ("\til y a eu des warnings");
      }
  }

}
